<?php



/**
 * Rest emitters notify registered observers using a POST request.
 * They are configured with an url to which they post the request.
 * Also, they don't wait for response.
 *
 * This emitter works fine when I signal my server from localhost. However,
 * it doesn't seem to work if I try to signal localhost from localhost
 * (wamp, win7).
 *
 * @author Benjamin
 */
class RestEmitter implements SignalEmitterI
{
  /**
   * @var RestEmitterConfigDTO
   */
  private $_config = null;

  public function setConfig( EmitterConfigDTO $config )
  {
    $this->_config = $config;
  }

  /**
   * Notifies the observer and immediatelly returns.
   * @param CEvent $event
   */
  public function emit(CEvent $event)
  {
    $this->postAsync( $this->_config->url, $event->params );
  }

  /**
   * Sends a post request using a socket. After the request has been written,
   * the socket will be closed. We don't wait for response.
   * 
   * @param string $url
   * @param array $params
   */
  protected function postAsync( $url, array $params )
  {
    $post_params = array();

    foreach ($params as $key => $val)
      $post_params[] = $key.'='.urlencode(json_encode($val));

    $post_string = implode( '&', $post_params );
    $parts = parse_url($url);

    $fp = fsockopen(
      $parts['host'],
      isset( $parts['port'] ) ? $parts['port'] : 80,
      $errno,
      $errstr,
      30
    );

    if ($fp)
    {
      $out = "POST ".$parts['path']." HTTP/1.1\r\n";
      $out.= "Host: ".$parts['host']."\r\n";
      $out.= "Content-Type: application/x-www-form-urlencoded\r\n";
      $out.= "Content-Length: ".strlen($post_string)."\r\n";
      $out.= "Connection: Close\r\n\r\n";

      if (isset($post_string))
        $out.= $post_string;

      fwrite( $fp, $out );
      fclose( $fp );
    }
  }

}
