<?php

class SignalController extends Controller
{
  public function actions()
  {
    return array(
      'wsdl'=>array(
        'class'=>'CWebServiceAction',
      ),
    );
  }

  /**
   * @return SignalDTO[] list of available signal ids
   * @soap
   */
  public function getSignals()
  {
    $module = $this->getModule();

    $signals = array();
    foreach ($module->signals as $signal)
    {
      $signalDTO = new SignalDTO();
      $signalDTO->id = $signal;

      $signals[] = $signalDTO;
    }
    
    return $signals;
  }

  /**
   * This method generates a connection key for a given url, and assiciates it
   * with the hostname portion of the url. The connection key is used to
   * connect and disconnect from signals. At any time, there can only be one
   * connection key associated with a given hostname.
   *
   * @param string $url for which a connection key shall be generated
   * @return string the key for the hostname
   * @soap
   */
  public function getConnectionKey( $url )
  {
    $model = new ARConnectionKeys( 'create' );
    $model->setUrl( $url );
    $model->key = md5( uniqid(mt_rand(), true) );

    if ($model->save())
      return $model->key;
    else
      throw new CException('Error on providing a connection key.');
  }

  /**
   * @param string $signalId to which you want to connect
   * @param string $url that shall receive notifications
   * @param string $connectionKeyPath path to the key file. Defaults to an
   *        empty string, which means that the key file won't be created in a
   *        specified folder.
   * @return bool if the connection has been established
   * @soap
   */
  public function connectToSignal( $signalId, $url, $connectionKeyPath='' )
  {
    // create the new signal emitter
    $model = new ARRestSignalEmitters( 'RestConnect' );
    $model->signalId = $signalId;
    $model->setUrl( $url );
    $model->setConnectionKeyPath( $connectionKeyPath );

    if ($model->save())
      return true;
    else
    {
      $aErrors = $model->getErrors();
      $errorDump = CVarDumper::dumpAsString($aErrors);
      throw new CException("Error on connecting to a signal: '$errorDump'");
    }
  }

  /**
   * @param string $signalId from which you want to disconnect
   * @param string $url to which the signal is connected
   * @param string $connectionKeyPath path to the key file. Defaults to an
   *        empty string, which means that the key file won't be created in a
   *        specified folder.
   * @return bool if the connection has been removed
   * @soap
   */
  public function disconnectFromSignal( $signalId, $url, $connectionKeyPath='' )
  {
    // find the signal emitter
    $aModels = ARRestSignalEmitters::model()->findAll(
      'type=:type AND signalId=:signalId AND hostname=:hostname',
      array(
        ':type'     => EmitterType::REST,
        ':signalId' => $signalId,
        ':hostname' => ARRestSignalEmitters::getHostname( $url ),
      )
    );

    /* @var $model ARRestSignalEmitters */
    $target = null;
    foreach ($aModels as $model)
    {
      $config = $model->getEmitterConfig();

      if ($config->url === $url)
      {
        $model->setConnectionKeyPath( $connectionKeyPath );
        $model->setScenario( 'RestDisconnect' );

        if ($model->validate())
        {
          $target = $model;
          break;
        }
        else
        {
          $aErrors = $model->getErrors();
          $errorDump = CVarDumper::dumpAsString($aErrors);
          throw new CException("Error on disconnecting from a signal: '$errorDump'");
        }
      }
    }

    if ($target instanceof ARRestSignalEmitters)
    {
      if ($target->delete())
        return true;
      else
        throw new CException("Error on disconnecting from a signal: can't delete record.");
    }
    else
      throw new CException("Error on disconnecting from a signal: didn't find a connection.");
  }

}