<?php

/**
 * Description of ARConnectionKeys
 *
 * @author Benjamin
 */
class ARConnectionKeys extends ConnectionKeys
{
  
  private $_url;
  
	/**
	 * Returns the static model of the specified AR class.
	 * @return ARSignalEmitters the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		return array_merge(parent::rules(), array(
			array('hostname', 'validateUrl', 'on'=>'create'),
			array('hostname', 'unique'),
		));
	}

  /**
   * Validates, that the provided url can be reached.
   */
  public function validateUrl( $attribute, $params )
  {
    $aUrlParts = parse_url( $this->_url );
    if (!is_array($aUrlParts))
    {
      $this->addError( 'hostname', "Invlaid: '{$this->_url}'." );
      return;
    }

    $scheme = $aUrlParts['scheme'];
    switch ($scheme)
    {
    case 'file':
        $filepath = $aUrlParts['path'];
        if (!file_exists($filepath))
          $this->addError( 'hostname', "File does not exist '$filepath'." );
        break;
    default:
        $headers = @get_headers( $this->_url );

        $ok = is_array( $headers );
        if ($ok)
          $ok = array_key_exists(0,$headers);
        if ($ok)
          $ok = $headers[0] === 'HTTP/1.1 200 OK' || $headers[0] === 'HTTP/1.0 200 OK';

        if (!$ok)
          $this->addError( 'hostname', "Can't reach '{$this->_url}'." );
    }
  }

  public function setUrl( $url )
  {
    $this->_url = $url;

    $aUrlParts = parse_url( $url );
    if (!is_array($aUrlParts))
    {
      $this->hostname = '';
      return;
    }

    $scheme = $aUrlParts['scheme'];
    switch ($scheme)
    {
    case 'file':
        $this->hostname = 'localhost';
        break;
    default:
        $this->hostname = array_key_exists('host',$aUrlParts) ? $aUrlParts['host'] : '';
        break;
    }
  }
}
