<?php



/**
 * Description of ARSignalEmitters
 *
 * @author Benjamin
 */
class ARSignalEmitters extends SignalEmitters
{

  /**
	 * Returns the static model of the specified AR class.
	 * @return ARSignalEmitters the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

  public function behaviors()
  {
    return array(
      'ECompositeUniqueKeyValidatable' => array(
        'class' => 'ECompositeUniqueKeyValidatable',
        'uniqueKeys' => array(
          'attributes' => 'signalId, hostname, filepath',
          'errorMessage' => 'The signal is already attached to the specified slot',
        )
      ),
    );
  }

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array_merge(parent::rules(), array(
			array('signalId', 'validateSignalId'),
			array('type', 'validateType'),
      array('*', 'compositeUniqueKeysValidator'),
		));
	}

  public function validateSignalId( $attribute, $params )
  {
    $signalsModule = $this->getSignalsModule();
    $signalId = $this->$attribute;
    if (!$signalsModule->signalExists($signalId))
      $this->addError( 'signalId', "Unknown signal id '$signalId'." );
  }

  public function validateType( $attribute, $params )
  {
    $type = $this->$attribute;
    
    try {
      CPropertyValue::ensureEnum($type, 'EmitterType');
    } catch (CException $exc) {
      $this->addError( 'type', $exc->getMessage() );
    }
  }

  /**
   * Validates composite unique keys
   *
   * Validates composite unique keys declared in the
   * ECompositeUniqueKeyValidatable bahavior
   */
  public function compositeUniqueKeysValidator() {
      $this->validateCompositeUniqueKeys();
  }

  /**
   * @return string
   */
  public function getSignalId()
  {
    return $this->signalId;
  }

  /**
   * @param string signalId
   */
  public function setSignalId( $signalId )
  {
    $this->signalId = $signalId;
  }

  /**
   * @return EmitterConfigDTO
   */
  public function getEmitterConfig()
  {
    return unserialize( $this->config );
  }

  /**
   * @param EmitterConfigDTO $config
   */
  public function setEmitterConfig( EmitterConfigDTO $config )
  {
    $this->config = serialize( $config );
  }

}
